﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using SysKobo.DBService;
using SysKobo.Stdlib;
using SysKobo.Controls;

namespace SampleUse.UForm
{
    public partial class Form2 : SysKobo.Forms.formViewBase
    {
        public string dbName = string.Empty;    //接続先のDB

        private Rectangle dragBoxFromMouseDown;      // 座標用
        private int rowIndexFromMouseDown;           // 移動元Index用
        private int rowIndexOfItemUnderMouseToDrop ; // 移動先Index用

        public Form2()
        {
            InitializeComponent();
        }

        private void Form2_Load(object sender, EventArgs e)
        {
            base.ViewHelper = new DataGridViewHelper(this, null, base.GetImageList());
            base.ViewHelper.dataGridView_RowPost(dataGridView1);

            this.CreateDBFile();
            this.GetData();
        }


        private void GetData()
        {
            string[] sql = new string[1];
            using (websqlcmd wSrv = dbsvex.SetDataServer(this, this.dbName))
            {
                //DBオープンできたか
                if (wSrv.ws_errorFlg)
                {
                    errinfo.MsgShow(this, wSrv);
                    return;
                }

                sql[0] = " select * from test order by jyun ";
                DataSet sSet = wSrv.fill(sql);
                if (wSrv.ws_errorFlg)
                {
                    errinfo.MsgShow(this, wSrv);
                    return;
                }

                dataGridView1.DataSource = sSet.Tables[0];

                foreach (DataGridViewColumn c in dataGridView1.Columns)
                {
                    c.SortMode = DataGridViewColumnSortMode.NotSortable;
                }
            }
        }

        private void Form2_Shown(object sender, EventArgs e)
        {
        }

        private bool CreateDBFile()
        {
            //DBオープン
            websqlcmd wSrv = new websqlcmd(this, dbsvex.SetDBParam(this, this.dbName), dbsvex.SetDBParam(this, this.dbName));
            if (dbsvex.IsServiceError(wSrv))
            {
                errinfo.MsgShow(this, wSrv);
                return false;
            }

            //DBが有効か確認する
            string[] sql = new string[1];
            sql[0] = " select name from sqlite_master where type = 'table' order by name ";

            DataSet dSet = wSrv.fill(sql);
            if (dbsvex.IsServiceError(wSrv))
            {
                errinfo.MsgShow(this, wSrv);
                return false;
            }
            foreach (DataRow dr in dSet.Tables[0].Rows)
            {
                if (dr[0].ToString() == "test") return true;
            }

            //DBが空なのでテーブルを作成する
            sql[0] = " create table [test] ( ";
            sql[0] += " [skey] integer not null unique, ";
            sql[0] += " [txt] varchar(50), ";
            sql[0] += " [jyun] integer, ";
            sql[0] += " primary key (skey) ";
            sql[0] += " ) ";

            wSrv.CreateTable(sql[0]);
            if (dbsvex.IsServiceError(wSrv))
            {
                errinfo.MsgShow(this, wSrv);
                return false;
            }

            return this.InsertDummyData();
        }

        private bool InsertDummyData()
        {
            string[] sql = new string[10];

            for (int i = 0; i < sql.Length; i++)
            {
                sql[i] = " insert into test ( skey, txt, jyun ) values ( ";
                sql[i] += string.Format("{0}", i + 1);
                sql[i] += string.Format(",'{0}'", this.GeneratePassword(10+i));
                sql[i] += string.Format(",{0}",i + 1);
                sql[i] += " ) ";
            }

            using (websqlcmd wSrv = dbsvex.SetDataServer(this,this.dbName))
            {
                //DBオープンできたか
                if (wSrv.ws_errorFlg)
                {
                    errinfo.MsgShow(this, wSrv);
                    return false;
                }

                //SQL実行
                wSrv.ExecuteNonQuery(sql);
                if (wSrv.ws_errorFlg)
                {
                    errinfo.MsgShow(this, wSrv);
                    return false;
                }
            }

            return true;

        }


        private static readonly string passwordChars = "0123456789abcdefghijklmnopqrstuvwxyz";

        private string GeneratePassword(int length)
        {
            StringBuilder sb = new StringBuilder(length);
            Random r = new Random();

            for (int i = 0; i < length; i++)
            {
                //文字の位置をランダムに選択
                int pos = r.Next(passwordChars.Length);
                //選択された位置の文字を取得
                char c = passwordChars[pos];
                //パスワードに追加
                sb.Append(c);
            }

            return sb.ToString();
        }

        private void dataGridView1_MouseMove(object sender, MouseEventArgs e)
        {
            // 左クリックの場合
            if (e.Button == MouseButtons.Left)
            {
                if (dragBoxFromMouseDown != Rectangle.Empty && !(dragBoxFromMouseDown.Contains(e.X, e.Y)))
                {
                    DragDropEffects dropEffect = dataGridView1.DoDragDrop(dataGridView1.Rows[rowIndexFromMouseDown], DragDropEffects.Move);
                }
            }
        }

        private void dataGridView1_MouseDown(object sender, MouseEventArgs e)
        {
            rowIndexFromMouseDown = dataGridView1.HitTest(e.X, e.Y).RowIndex;

            // ヘッダー以外
            if (rowIndexFromMouseDown > -1)
            {
                var dragSize = SystemInformation.DragSize;
                // ドラッグ操作が開始されない範囲を取得
                dragBoxFromMouseDown = new Rectangle(new Point(e.X - dragSize.Width / 2, e.Y - dragSize.Height / 2), dragSize);
            }
            else
            {
                dragBoxFromMouseDown = Rectangle.Empty;
            }
        }

        private void dataGridView1_DragOver(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.Move;
        }

        private void dataGridView1_DragDrop(object sender, DragEventArgs e)
        {
            // データグリッドのポイントを取得
            Point clientPoint = dataGridView1.PointToClient(new Point(e.X, e.Y));
            // 移動先INDEX
            rowIndexOfItemUnderMouseToDrop = dataGridView1.HitTest(clientPoint.X, clientPoint.Y).RowIndex;

            // ドラッグ＆ドロップ効果【移動】の場合・INDEX範囲内の場合
            if (e.Effect == DragDropEffects.Move &&
               rowIndexOfItemUnderMouseToDrop > -1)
            {
                // 移動データ退避
                DataTable Dt = (DataTable)dataGridView1.DataSource;
                Object[] rowArray = Dt.Rows[rowIndexFromMouseDown].ItemArray;
                DataRow row = Dt.NewRow();
                row.ItemArray = rowArray;

                // 移動元削除
                Dt.Rows.RemoveAt(rowIndexFromMouseDown);
                // 移動先新規行挿入
                Dt.Rows.InsertAt(row, rowIndexOfItemUnderMouseToDrop);
            }
        }

        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void toolStripButton2_Click(object sender, EventArgs e)
        {
            string[] sql = new string[1];

            using (websqlcmd wSrv = dbsvex.SetDataServer(this, this.dbName))
            {
                //DBオープンできたか
                if (wSrv.ws_errorFlg)
                {
                    errinfo.MsgShow(this, wSrv);
                    return;
                }

                DataTable Dt = (DataTable)dataGridView1.DataSource;

                for (int i = 0; i < Dt.Rows.Count; i++)
                {
                    DataRow dr = Dt.Rows[i];

                    if (i + 1 != Tool.atoi(dr["jyun"].ToString()))
                    {
                        sql[0] = " update test set ";
                        sql[0] += " jyun = " + (i + 1).ToString();
                        sql[0] += " where skey = " + dr["skey"].ToString();

                        wSrv.ExecuteNonQuery(sql);
                        if (wSrv.ws_errorFlg)
                        {
                            errinfo.MsgShow(this, wSrv);
                            return;
                        }
                    }
                }

            }

            //データの再取得
            this.GetData();

        }

        private void toolStripButton3_Click(object sender, EventArgs e)
        {
            using (SysKobo.Forms.formCustom form = new SysKobo.Forms.formCustom())
            {
                form.form_Init(base.iniFile.iniFileName);

                if (form.ShowDialog() == DialogResult.OK)
                {

                }
            }

        }

    }
}
